/*******************************************************************************
*	Packet handler for localizing and anchor node. command.h
*	Last Modified: 21/May/2009
*
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#ifndef COMMAND_H
#define COMMAND_H

#include <pthread.h>
#include "network.h"

//
#define GPIORED "/proc/gpio/gpio103"
#define GPIOGREEN "/proc/gpio/gpio104"
#define GPIOBLUE "/proc/gpio/gpio105"
#define BUF_SIZE 4
#define GPIO_CONTROL "/proc/gpio/GPCTL"

//
#define MAX_SEQ 3
// command numbers
#define BLUE_TOGGLE 11
#define BLUE_ON 12
#define BLUE_OFF 13
#define RED_TOGGLE 21
#define RED_ON 22
#define RED_OFF 23
#define GREEN_TOGGLE 31
#define GREEN_ON 32
#define GREEN_OFF 33
#define ALL_TOGGLE 41
#define ALL_ON 42
#define ALL_OFF 43
// localization commands
#define ASSIGN_XY  110
#define	LOC_DATA  111
#define EST_LOC_REPORT  113
#define LOC_OFF  116
#define ACTUAL_LOC_REPORT  118
#define PROPOGATE_ON 145
#define PROPOGATE_OFF 146
#define LOC_ON  200
#define CENTROID 201
#define WEIGHTEDCENTROID 202
#define GRIDSCORE 203
#define GRADIENT 204
#define WEIGHTED_SCORE 205
#define WEIGHTED_GRADIENT 206
// extra commands
#define RESTART 100
#define CHANGE_MAX_RSSI 101
#define CHANGE_LOCAL_TIMER_INTERVAL 102

/* this function initializes the sequence # list */
int init_seq_list();

/* packet handler */
int pkt_handler(packet *in_pkt);

/* command handler */
int cmd_handler(packet *in_pkt);

/* forward packet to base */
int forward_to_base(packet *out_pkt, int ACK_en);

/* forward pakcet to nodes */
int forward_to_node(packet *out_pkt, int ACK_en);

/* toggle green LED */
int toggle_green();

/* turn on green LED */
int on_green();

/* turn off green LED */
int off_green();

/* toggle red LED */
int toggle_red();

/* turn on red LED */
int on_red();

/* turn off red LED */
int off_red();

/* toggle blue LED */
int toggle_blue();

/* turn on blue LED */
int on_blue();

/* turn off blue LED */
int off_blue();

/* open socket for GUI */
void *openSocket(unsigned char *cmd_list);

/* send list of command to sensor nodes  */
//int send_cmd(unsigned char *cmd_list);

#endif
