/*******************************************************************************
*	Main function for base node. handler_base.c
*	Last Modified: 27/May/2009
*
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#include "network.h"
#include "command_base.h"
#include "router.h"
#include "localization.h"
#include "socket.h"
#include "util.h"

//
#define BUFFER_LEN 512
// debug switch
#define handler_debug 1

// control flags
int sendCommand = 0;	// flag for sending commands
int doneSending = 0;	// localization flag
pthread_mutex_t flag_mutex = PTHREAD_MUTEX_INITIALIZER;	// flag mutex

int main() {
	__u16 localAddr;	// local address
	unsigned char cmd_list[BUFFER_LEN];	// list of cmd to be sent
	packet in_pkt;	// received packet
	pthread_t routing_beacon_thread, cmdSocket_thread, timer_thread, router_thread, handler_thread;
	
	// welcome msg
	printf("This program receives packets from other imotes.\n");
	
	// set random number seed at the beginning of program
	srand(time(NULL));
	
	// initialize incoming pakcet
	initPkt(&in_pkt, 0);
	
	// initialize radio and set radio power level
	if (cc2420_init(POWER_LEVEL) < 0) {
		printf("Radio initialization error... \n");
		exit(-1);
	}

	// set local Address
	set_addr();
	
	// initializes sequence # list
	init_seq_list();
	
	// initialize routing table
	initTable();
	
	// initialize join list
	init_joinList();
	
	// read local address
	localAddr = get_addr();
	
	// initialize localization config file
	init_local_config();
	
	// open socket for GUI
	if (check_base()) {
		pthread_create(&cmdSocket_thread, NULL, (void*)&cmdSocket, (void*)cmd_list);
		//pthread_join(cmdSocket_thread, NULL);
	}
	
	// display thread to show the base-to-node routing table
	pthread_create(&router_thread, NULL, (void*)&display_routing_table, NULL);
	
	// thread sending out routing packets
	pthread_create(&routing_beacon_thread, NULL, (void*)&routingBeacon, NULL);
	
	// timer for localization algorithms
	pthread_create(&timer_thread, NULL, (void*)&local_timer, NULL);
	
	// main loop
	while (1) {
		// receive packets
		if (!sendCommand) {
			if (h_recvPkt(localAddr, &in_pkt, 1) == 0) {
				if (handler_debug) {
					printf("\nreceving data is: %s\n", in_pkt.data);
					printf("packet port number is: %d\n", in_pkt.type);
					printf("packet is from: %x\n", in_pkt.srcAddr);
					printf("packet sequence number is (in HEX): %x\n\n", in_pkt.seq);
				}
				// pass recieved packet to handler
				pthread_create(&handler_thread, NULL, (void*)&pkt_handler, (void*)&in_pkt);
				pthread_join(handler_thread, NULL);
				//pkt_handler(&in_pkt);
			}
		}
		else {
			// send command to nodes
			printf("\nSending out command.\n\n");
			// send cmd to nodes
			send_cmd(cmd_list);
			// set ctrl flags
			pthread_mutex_lock(&flag_mutex);
			sendCommand = 0;	// reset flag
			doneSending = 1;	// set flag
			pthread_mutex_unlock(&flag_mutex);
		}
		// join threads
		//join_threads();
	}
	return 0;
}
