/*******************************************************************************
*	Localization Algorithms. localization.c
*	Last Modified: 27/May/2009
*
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#include "command.h"
#include "router.h"
#include "localization.h"
#include "socket.h"

#define local_debug 0


beacon b[MAX_BEACONS];	// list of reach-able nodes
int resume_localization = 0;	// flag for resuming localization, if set program will resume from previous settings after crashed
int propagate = 0;		// propagation flag
int loc_status = 0;		// localization flag
int anchor = 0;			// anchor flag
float assignedX = 0;	// assigned X location
float assignedY = 0;	// assigned Y location
float estX = 0;			// estimated X location
float estY = 0;			// estimated Y location
int iteration_count = 0;	// # of iterations for localization
int neighborListIndex = 0;	// # of neighbors node can hear
int reset_b_list = 1;	// flag for resetting b list
//
int MAX_RSSI = 6;		// max sample period (NOTICE must be less than MAX_RSSI_BEACON)
int LOCAL_TIMER_INTERVAL = 2;	// localization timer inteval

/*
struct info
{
	int propagate;		// propagation flag
	int loc_status;		// localization flag
	int anchor;			// anchor flag
	float assignedX;	// assigned X location
	float assignedY;	// assigned Y location
	float estX;			// estimated X location
	float estY;			// estimated Y location
	int iteration_count;// # of iterations for localization
	int neighborListIndex;	// # of neighbors node can hear
} node_info = {0,0,0,0,0,0,0,0,0};
*/


/*
 * update localization config file
 */
int update_local_config()
{
	FILE *fp;
	// write local config to file
	fp = fopen("/root/local.conf", "w");
	if (fp == NULL)
		return -1;
	else {
		// write current config to file
		fprintf(fp, "%d\n", resume_localization);
		fprintf(fp, "%d\n", propagate);
		fprintf(fp, "%d\n", loc_status);
		fprintf(fp, "%d\n", anchor);
		fprintf(fp, "%f\n", assignedX);
		fprintf(fp, "%f\n", assignedY);
		fprintf(fp, "%f\n", estX);
		fprintf(fp, "%f\n", estY);
		fprintf(fp, "%d\n", MAX_RSSI);
		fprintf(fp, "%d\n", LOCAL_TIMER_INTERVAL);
		fclose(fp);
	}
	return 0;
}


/*
 * initializes local config file
 */
int init_local_config()
{
	FILE *fp;
	
	// open local config file
	fp = fopen("/root/local.conf", "r");
	if (fp == NULL) {
		if (!check_base()) {
			printf("local.conf doesn't exist, creating...\n");
			update_local_config();	// write default values to config file
		}
		return 0;
	}
	else {
		fscanf(fp, "%d\n", &resume_localization);
		// load previously crashed config
		if (resume_localization) {
			printf("Recovering from crash!\n");
			fscanf(fp, "%d\n%d\n%d\n%f\n%f\n%f\n%f\n%d\n%d\n", &propagate, &loc_status, &anchor, &assignedX, &assignedY, &estX, &estY, &MAX_RSSI, &LOCAL_TIMER_INTERVAL);
			fclose(fp);
			if (local_debug) {
				printf("\npropagate = %d\n", propagate);
				printf("loc_status = %d\n", loc_status);
				printf("anchor = %d\n", anchor);
				printf("assignedX = %f\n", assignedX);
				printf("assignedY = %f\n", assignedY);
				printf("estX = %f\n", estX);
				printf("estY = %f\n", estY);
				printf("MAX_RSSI = %d\n", MAX_RSSI);
				printf("LOCAL_TIME_INTERVAL = %d\n", LOCAL_TIMER_INTERVAL);
			}
		}
		else
			fclose(fp);
	}
	return 0;
}


/*
 * handler for localization packets 
 */
void localization_msg(packet* in_pkt)
{
	int i;
	__u16 sender_addr;
	
	if (local_debug)
		printf("\nLocalization handler started!\n");
	
	// change MAX_RSSI
	if (in_pkt->data[1] == CHANGE_MAX_RSSI) {
		MAX_RSSI = in_pkt->data[4];		// MAX_RSSI is unsigned char, in data[4]
		update_local_config();	// update config file
	}
	// change LOCAL_TIME_INTERVAL
	if (in_pkt->data[1] == CHANGE_LOCAL_TIMER_INTERVAL) {
		LOCAL_TIMER_INTERVAL = in_pkt->data[4];	// LOCAL_TIME_INTERVAL is unsigned char, in data[4]
		update_local_config();	// update config file
	}
	// turn on/off propagation 
	if (in_pkt->data[1] == PROPOGATE_ON) {
		propagate = 1;
		update_local_config();	// update local config file
	}
	if (in_pkt->data[1] == PROPOGATE_OFF) {
		propagate = 0;
		update_local_config();	// update local config file
	}
	// report location to GUI, only report on base nodes
	if (((in_pkt->data[1] == ACTUAL_LOC_REPORT) || (in_pkt->data[1] == EST_LOC_REPORT)) && check_base()) {
		printf("Reporing location to GUI!\n\n");
		dataSocket(*in_pkt);
	}
	// turn on locationing 
	if (in_pkt->data[1] >= LOC_ON) {
		loc_status = in_pkt->data[1];
		resume_localization = 1;	// set resume flag
		update_local_config();	// update local config file
		return;
	}
	// Turn off locationing
	if (in_pkt->data[1] == LOC_OFF) {
		loc_status = 0;
		iteration_count = 0;
		anchor = 0;
		assignedX = 0;
		assignedY = 0;
		estX = 0;
		estY = 0;
		propagate = 0;
		reset_b_list = 1;	// reset b list
		resume_localization = 0;	// do not resume if locationing is stopped
		update_local_config();	// update local config file
		off_green();	// turn-off green LED
		off_red();		// turn-off red LED
		return;
	}
	// anchor the mote at an x,y location
	// sets the location and turn off the timer
	if(in_pkt->data[1] == ASSIGN_XY) {
		memcpy(&assignedX, in_pkt->data + 4, 4);	// assign X location
		memcpy(&assignedY, in_pkt->data + 8, 4);	// assign Y location
		anchor = 1;		// set anchor flag
		resume_localization = 1;	// set resume flag
		//iteration_count = MAX_RSSI;		//
		update_local_config();	// update local config file
		if (local_debug)
			printf("\nAnchored at X = %f, Y = %f\n\n", assignedX, assignedY);
	}
	//
	if (loc_status) {
		sender_addr = in_pkt->srcAddr;	// get sender address
		// process received locationing packets from anchors
		if (in_pkt->data[1] == LOC_DATA && !anchor) {
			if (local_debug)
				printf("Location received from: %x\n", sender_addr);
			for (i = 0; i < MAX_BEACONS; i++) {
				// updata existing entry
				if (b[i].addr == sender_addr) {
					// sanity check to make sure not getting outside boundaries
					if (b[i].RSSIindex < MAX_RSSI) {
						// assign x location
						memcpy(&(b[i].x), (in_pkt->data + 4), 4);
						// assign y location
						memcpy(&(b[i].y), (in_pkt->data + 8), 4);
						// rssi
						b[i].RSSI[b[i].RSSIindex] = in_pkt->rss;
						b[i].RSSIindex++;
						//
						if (local_debug) {
							printf("\nLocation data received from %x, X: %f, Y: %f\n", sender_addr, b[i].x, b[i].y);
							printf("RSSI is: %d\n\n", b[i].RSSI[b[i].RSSIindex]);
						}
						break;
					}
				}
				// addes new entry
				else if (b[i].addr == 0) {
					if (local_debug)
						printf("Neighbor added: %x\n", sender_addr);
					b[i].RSSIindex = 0;
					b[i].addr = sender_addr;
					// assign x location
					memcpy(&(b[i].x), (in_pkt->data + 4), 4);
					// assign y location
					memcpy(&(b[i].y), (in_pkt->data + 8), 4);
					// rss
					b[i].RSSI[b[i].RSSIindex] = in_pkt->rss;
					b[i].RSSIindex++;
					// update # of neighbors one can hear
					neighborListIndex++;
					//
					if (local_debug) {
						printf("\nLocation data received from %x, X: %f, Y: %f\n", sender_addr, b[i].x, b[i].y);
						printf("RSSI is: %d\n\n", b[i].RSSI[b[i].RSSIindex]);
					}
					break;
				}
			}
		}
	}
	return;
}


/*
 * timer for localization alrogithms
 * fires every LOCAL_TIMER_INTERVAL seconds. It runs the localization algorithms
 * and sends out location information based on current settings/commands of the
 * node.
 */
void *local_timer()
{
	packet out_pkt;						// out going localization pkt
	unsigned char location_data[12];	// location data
	__u16 localAddr;					// local node address
	float tempRSSI;
	int i, j;
	time_t preTime, curTime;	// timer (in seconds)
	grid g;				// grid for gird score
	pathLossModel p;	// path loss model for weighted grid score
	
	// get local address
	localAddr = get_addr();
	
	// set initial prevTime
	preTime = time(NULL);
	
	// main loop
	while (1) {
		curTime = time(NULL);	// get current system time
		if (curTime >= preTime + LOCAL_TIMER_INTERVAL) {
			//update previous time
			preTime = curTime;
			// debug display
			if (local_debug) {
				printf("\nLocalization Timer!\n");
				printf("Current time is %ld\n", curTime);
				printf("iteration_count is %d\n", iteration_count);
				printf("neighborListIndex is %d\n\n", neighborListIndex);
			}
			// initialize b list
			if (reset_b_list) {
				for (i = 0; i < MAX_BEACONS; i++) {
					b[i].RSSIindex = 0;
					b[i].addr = 0;
				}
				reset_b_list = 0;	// reset flag
			}

			// all that the base node does is blink green & blue
			if (check_base()) {
				toggle_green();
				toggle_blue();
				//sleep(LOCAL_TIMER_INTERVAL);
				continue;
			}
			// do nothing if localizing is turned off
			if (!loc_status) {
				// wait till next run
				//sleep(LOCAL_TIMER_INTERVAL);
				continue;
			}
			// for anchor nodes
			if (anchor) {
				iteration_count++;	// update counter
				toggle_green();		// toggle green LED for anchor node
				// build location packet
				initPkt(&out_pkt, ANCHOR_BROADCAST);	// pkt initialization
				out_pkt.data[0] = 0;	// set rand to 0
				out_pkt.data[1] = LOC_DATA;
				memcpy(out_pkt.data + 2, &localAddr, 2);	// local address
				memcpy(out_pkt.data + 4, &assignedX, 4);	// x location
				memcpy(out_pkt.data + 8, &assignedY, 4);	// y location
				// broadcast location packet, no ACK
				printf("\nBroadcasting anchor location!\n");
				h_send(0xffff, &out_pkt, 0);
				// send location data back to base every MAX_WAIT * LOCAL_TIMER_INTERVAL sec
				if (iteration_count > MAX_WAIT) {
					iteration_count = 0;	// reset iteration_count
					out_pkt.type = TO_BASE_DATA_PKT_TYPE;	// change type
					out_pkt.seq = rand();				// assign new seq number
					out_pkt.data[1] = ACTUAL_LOC_REPORT;// report actual location
					sendtoBase(&out_pkt, 1);	// report actual location back to base
					printf("\nSending location data back to base.\n\n");
				}
				continue;
			}
			// for localizing nodes
			else {
				toggle_red();	// blink red for localizing node
				//if (iteration_count >= MAX_RSSI) {
				if ((iteration_count >= MAX_RSSI) && (neighborListIndex != 0)) {
					if (local_debug)
						printf("Numbers of neighbors = %d\n", neighborListIndex);
					for (i = 0; i < neighborListIndex; i++) {
						// total up the RSSI measurements to get an average
						tempRSSI = 0;
						for (j = 0; j < b[i].RSSIindex; j++)
							tempRSSI += b[i].RSSI[j];
						// calculate average RSSI values obtained from anchor
						tempRSSI = tempRSSI / b[i].RSSIindex;
						// calculate the distance to that anchor
						b[i].d = RSSI_to_distance(tempRSSI, K, ALPHA);
						if (local_debug)
							printf("Distance = %f \tRSSI: %f From node: %x\n", b[i].d, tempRSSI, b[i].addr);
					}
					iteration_count = 0;	// reset iteration counter
					// run centroid
					if(loc_status == CENTROID) {
						printf("\nComputing Location using Centroid method.\n");
						centroid(b, neighborListIndex, &estX, &estY);
						if (local_debug)
							printf("Result of centroid is: X = %f, Y = %f\n", estX, estY);
						// build location packet
						initPkt(&out_pkt, TO_BASE_DATA_PKT_TYPE);	// pkt initialization
						out_pkt.data[0] = 0;	// set rand to 0
						out_pkt.data[1] = EST_LOC_REPORT;
						memcpy(out_pkt.data + 2, &localAddr, 2);// local address
						memcpy(out_pkt.data + 4, &estX, 4);		// x location
						memcpy(out_pkt.data + 8, &estY, 4);		// y location
						// report estimated location back to base
						sendtoBase(&out_pkt, 1);
						// update local config file
						update_local_config();	
					}
					// run weighted centroid
					else if (loc_status == WEIGHTEDCENTROID) {
						printf("\nComputing Location using Weighted Centroid method.\n");
						weightedCentroid(b, neighborListIndex, &estX, &estY);
						if (local_debug)
							printf("Result of centroid is: X = %f, Y = %f\n", estX, estY);
						// build location packet
						initPkt(&out_pkt, TO_BASE_DATA_PKT_TYPE);	// pkt initialization
						out_pkt.data[0] = 0;	// set rand to 0
						out_pkt.data[1] = EST_LOC_REPORT;
						memcpy(out_pkt.data + 2, &localAddr, 2);// local address
						memcpy(out_pkt.data + 4, &estX, 4);		// x location
						memcpy(out_pkt.data + 8, &estY, 4);		// y location
						// report estimated location back to base
						sendtoBase(&out_pkt, 1);
						// update local config file
						update_local_config();	
					}
					// run gridscore
					else if (loc_status == GRIDSCORE) {
						printf("\nComputing Location using Grid Score method.\n");
						weightedCentroid(b, neighborListIndex, &estX, &estY);
						if (local_debug)
							printf("Result of Weighted-Cetroid in Gridscore is: X = %f, Y = %f\n", estX, estY);
						// setup struct
						g.xmax = estX + GRID_SIZE;
						g.xmin = estX - GRID_SIZE;
						g.ymax = estY + GRID_SIZE;
						g.ymin = estY - GRID_SIZE;
						g.stepp = GRID_STEP;
						voting(b, neighborListIndex, g, 2.5, &estX, &estY);
						if (local_debug)
							printf("Result of Gridscore is: X = %f, Y = %f\n", estX, estY);
						// build location packet
						initPkt(&out_pkt, TO_BASE_DATA_PKT_TYPE);	// pkt initialization
						out_pkt.data[0] = 0;	// set rand to 0
						out_pkt.data[1] = EST_LOC_REPORT;
						memcpy(out_pkt.data + 2, &localAddr, 2);// local address
						memcpy(out_pkt.data + 4, &estX, 4);		// x location
						memcpy(out_pkt.data + 8, &estY, 4);		// y location
						// report estimated location back to base
						sendtoBase(&out_pkt, 1);
						// update local config file
						update_local_config();	
					}
					// run gradient
					else if (loc_status == GRADIENT) {
						printf("\nComputing Location using Gradient method.\n");
						gradient(b, neighborListIndex, DELTA, &estX, &estY);
						if (local_debug)
							printf("Result of Gradient is: X = %f, Y = %f\n", estX, estY);
						// build location packet
						initPkt(&out_pkt, TO_BASE_DATA_PKT_TYPE);	// pkt initialization
						out_pkt.data[0] = 0;	// set rand to 0
						out_pkt.data[1] = EST_LOC_REPORT;
						memcpy(out_pkt.data + 2, &localAddr, 2);// local address
						memcpy(out_pkt.data + 4, &estX, 4);		// x location
						memcpy(out_pkt.data + 8, &estY, 4);		// y location
						// report estimated location back to base
						sendtoBase(&out_pkt, 1);
						// update local config file
						update_local_config();	
					}
					// run weighted grid score
					else if (loc_status == WEIGHTED_SCORE) {
						printf("\nComputing Location using Weighted Grid Score method.\n");
						weightedCentroid(b, neighborListIndex, &estX, &estY);
						if (local_debug)
							printf("Result of Weighted-Cetroid in Gridscore is: X = %f, Y = %f\n", estX, estY);
						// setup grid
						g.xmax = estX + GRID_SIZE;
						g.xmin = estX - GRID_SIZE;
						g.ymax = estY + GRID_SIZE;
						g.ymin = estY - GRID_SIZE;
						g.stepp = GRID_STEP;
						// setup path loss model
						p.alpha = ALPHA;
						p.k = K;
						p.sigdB = SIGDB;
						// run weighted grid score
						w_voting(b, neighborListIndex, g, p, &estX, &estY);
						if (local_debug)
							printf("Result of Weighted Grid Score is: X = %f, Y = %f\n", estX, estY);
						// build location packet
						initPkt(&out_pkt, TO_BASE_DATA_PKT_TYPE);	// pkt initialization
						out_pkt.data[0] = 0;	// set rand to 0
						out_pkt.data[1] = EST_LOC_REPORT;
						memcpy(out_pkt.data + 2, &localAddr, 2);// local address
						memcpy(out_pkt.data + 4, &estX, 4);		// x location
						memcpy(out_pkt.data + 8, &estY, 4);		// y location
						// report estimated location back to base
						sendtoBase(&out_pkt, 1);
						// update local config file
						update_local_config();	
					}
					// run weighted gradient
					else if (loc_status == WEIGHTED_GRADIENT) {
						printf("\nComputing Location using Weighted Gradient method.\n");
						weightedCentroid(b, neighborListIndex, &estX, &estY);
						if (local_debug)
							printf("Result of Weighted-Cetroid in Gridscore is: X = %f, Y = %f\n", estX, estY);
						// run weighted gradient
						w_gradient(b, neighborListIndex, DELTA, &estX, &estY);
						if (local_debug)
							printf("Result of Weighted Gradient is: X = %f, Y = %f\n", estX, estY);
						// build location packet
						initPkt(&out_pkt, TO_BASE_DATA_PKT_TYPE);	// pkt initialization
						out_pkt.data[0] = 0;	// set rand to 0
						out_pkt.data[1] = EST_LOC_REPORT;
						memcpy(out_pkt.data + 2, &localAddr, 2);// local address
						memcpy(out_pkt.data + 4, &estX, 4);		// x location
						memcpy(out_pkt.data + 8, &estY, 4);		// y location
						// report estimated location back to base
						sendtoBase(&out_pkt, 1);
						// update local config file
						update_local_config();	
					}
				}
				// reset counter if didn't pick up anything
				else if ((iteration_count >= MAX_RSSI) && (neighborListIndex == 0))
					iteration_count = 0;
				// reset list
				if (iteration_count == 0) {
					neighborListIndex = 0;	// zero out neighnbor list
					for( i = 0; i < MAX_BEACONS; i++) {
						b[i].addr = 0;
						b[i].RSSIindex = 0;
					}
				}
				// update iteration count
				iteration_count++;
				// propogate location
				if (propagate && estX && estY) {
					// build location packet
					out_pkt.data[0] = 0;	// set rand to 0
					out_pkt.data[1] = LOC_DATA;
					memcpy(out_pkt.data + 2, &localAddr, 2);	// local address
					memcpy(out_pkt.data + 4, &estX, 4);	// estimated x location
					memcpy(out_pkt.data + 8, &estY, 4);	// estimated y location
					// send location packet, no ACK
					initPkt(&out_pkt, ANCHOR_BROADCAST);	// pkt initialization
					h_send(0xffff, &out_pkt, 0);
				}
			}
			// wait till next run
			//sleep(LOCAL_TIMER_INTERVAL);
		}
	}
	// terminate thread
	pthread_exit(NULL);
}


/*******************************************************************************
*   RSSI2Distance Converts RSSI measurements into distance estimates
*   Copyright (C) 2009  Jeffrey Vander Stoep jevander@u.washington.edu
*
*   This program is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*   d = (k/10^(RSS/10))^(1/alpha)
*   input RSSI, k and alpha.  See CC2420 datasheet on converting RSSI
*   to received power.
* *****************************************************************************/
float RSSI_to_distance(float RSSI, float k, float alpha)
{
	float d = 0.0;
	float Pr = 0.0;
	float Pr_dBm = 0.0;
	Pr_dBm = (RSSI - 45.0);		// As per the CC2420 data sheet from RSSI to dBm
	Pr = pow(10.0, Pr_dBm/10);	// From dBm to watts
	//Pt = Pt_t[Transmit_level];
	d = pow(k/Pr, 1.0/alpha);
	return(d);
}

/*******************************************************************************
* Title: centroid
* Description: This localization algorithm estimates location
*        as the center of gravity of the beacons within radio range.
*        Does not use RSSI measurements.
* Author: Jeff Vander Stoep jvanderstoep@gmail.com
* *****************************************************************************/
void centroid(beacon b[MAX_BEACONS], int num_b, float *estX, float *estY)
{
	int i;
	float x = 0.0;
	float y = 0.0;

	for(i = 0; i < num_b; i++)
	{
		x = x + (b[i].x);
		y = y + (b[i].y);
	}
	x = x / num_b;
	y = y / num_b;
	*estX = x;
	*estY = y;
	return;
}

/*******************************************************************************
* Title: weightedCentroid
* Description: This localization algorithm estimates location as the center of
*        gravity of the beacons within radio range, weighting each beacons
*        location (or its gravitational pull) by the inverse of the estimated
*        distance to that beacon. Distance estimates can be done using a
*        RSSI and a pathloss model.
* Author: Jeff Vander Stoep jvanderstoep@gmail.com
* Here is whats going on. Three different distances are estimated (d1, d2, d3)
*      along with the coordinates (x1, x2, x3) and (y1, y2, y3) the three
*      beacons. The x coordinate is calculated using as
*      x = ((1/d1)*x1 + (1/d2)*x2 + (1/d3)*x3) * (1/(1/d1 + 1/d2 + 1/d3))
*      where  (1/(1/d1 + 1/d2 + 1/d3)) normalizes the weighting
* *****************************************************************************/
void weightedCentroid(beacon b[MAX_BEACONS], int num_b, float *estX, float *estY)
{
   int i;
   float x = 0;
   float y = 0;
   float total_weight = 0;

   for(i = 0; i < num_b; i++ )
   {
	   //add up the 1/d multiples to be used for normalizing
	   total_weight = total_weight + 1/b[i].d;
	   x = x + ((b[i].x) / b[i].d);
	   y = y + ((b[i].y) / b[i].d);
   }
   *estX = ((1 / total_weight) * x);
   *estY = ((1 / total_weight) * y);
   return;
}

/*******************************************************************************
* Title: gradient
*
* Description: This is the gradient descent algorithm. Computes the
*        least-mean-square (LMS), then the derivative of the LMS.
*        Finally it moves towards minimizing the error by moving
*        towards the minimum of the gradient by delta*gradient.
* Author: Jeffrey Vander Stoep jevander@u.washington.edu
*
*******************************************************************************/
void gradient(beacon b[MAX_BEACONS], int num_b, float delta, float * estX, float * estY) //, uint8_t nodeId)
{
	int i,j;
	static int IterationNumber;
	float oldX, oldY;
	float x,y;
	float tempX, tempY;
	float Xgradient = 0, Ygradient = 0;
	float neighborX, neighborY;
	float computedDistance;
	float mindistance; //for initializing to nearest neighbor
	int mindist_index = 0;
	if ((!(*estX)) || (!(*estY))) IterationNumber = 0;
	if (IterationNumber ==0)
	{
		mindistance = b[0].d;
		for (i = 1; i< num_b; i++)
		{
			if (b[i].d < mindistance)
			{
				mindistance = b[i].d;
				mindist_index = i;
			}
		}
		x = b[mindist_index].x + 0.1;
		y = b[mindist_index].y + 0.1;
		IterationNumber++;
	}
	else
	{
		oldX = *estX;
		oldY = *estY;
		for(i=0;i < num_b; i++)
		{
			neighborX = b[i].x;
			neighborY = b[i].y;

			computedDistance = sqrt(pow((oldX - neighborX),2) + pow((oldY - neighborY),2));
			Xgradient = Xgradient + (computedDistance - b[i].d)* ((oldX - neighborX))/computedDistance;
			Ygradient = Ygradient + (computedDistance - b[i].d)* ((oldY - neighborY))/computedDistance;
		}
		tempX = ((oldX - (delta * Xgradient)));// > 0);// ? (oldX - (ALPHA * Xgradient)) : 0;
		tempY = ((oldY - (delta * Ygradient)));// > 0);// ? (oldY - (ALPHA * Ygradient)) : 0;

		x = tempX;
		y = tempY;
	}
	*estX = x;
	*estY = y;
}

/*******************************************************************************
* Title: voting - basic
* Description: This is a primitive algorithm for grid scoring with the method of intersecting
* annulli. It is not optimised for speed/memory.
* Author: Jeff Vander Stoep
* *****************************************************************************/
void voting(beacon b[MAX_BEACONS], int num_b, grid g, float errorThreshold, float *estX, float *estY)
{
	int i;
	int presentIndex;
	float x,y;
	int score;
	int MaxScore = 0;
	int MaxScoreIndex = 0;
	int Xincrement = 0,Yincrement = 0;
	float computedDistance;
	float minDistanceBound, maxDistanceBound;
	float *XofCellsWithMaxScore;
	float *YofCellsWithMaxScore;
	int k = 0;
	int xlen, ylen;
	xlen = floor((g.xmax-g.xmin)/g.stepp+1);
	ylen = floor((g.ymax-g.ymin)/g.stepp+1);
	if ((XofCellsWithMaxScore = (float*) malloc(sizeof(float)*xlen*ylen+1)) == NULL)
		printf("X malloc failed\r\n");
	if ((YofCellsWithMaxScore = (float*) malloc(sizeof(float)*xlen*ylen+1)) == NULL)
		printf("Y malloc failed\r\n");
	for (x = g.xmin; x <= g.xmax; x += g.stepp)
	{
		for (y = g.ymin; y <= g.ymax; y += g.stepp)
		{
			score = 0;
			for (i = 0; i < num_b; i++)
			{
				computedDistance = sqrt(pow((x - b[i].x),2) + pow((y - b[i].y),2));
				minDistanceBound = ((b[i].d - errorThreshold) > 0) ? (b[i].d - errorThreshold) : 0;
				maxDistanceBound = b[i].d + errorThreshold;
				if((minDistanceBound <= computedDistance) && (computedDistance <= maxDistanceBound ))
				{
					score+= 1;
				}
			}
			if (score >= MaxScore)
			{
				if (score > MaxScore)
				{
					MaxScore = score;
					MaxScoreIndex = 0;
				}
				if (MaxScore)
				{
					XofCellsWithMaxScore[MaxScoreIndex] = x;
					YofCellsWithMaxScore[MaxScoreIndex] = y;
					MaxScoreIndex++;
				}
			}
		}
	}
	x = 0;
	y = 0;
	for( i = 0; i < MaxScoreIndex; i++)
	{
		x += XofCellsWithMaxScore[i]/MaxScoreIndex;
		y += YofCellsWithMaxScore[i]/MaxScoreIndex;
	}
	*estX = x;
	*estY = y;
	free(XofCellsWithMaxScore);
	free(YofCellsWithMaxScore);
}

/********************************************************************************************
* Title: weighted voting (gird score)
* Description: This is a weighted version of voting method. Here, both
* errorThreshold and vote are weighted.
* annulli.
* Author: Jeff Vander Stoep
* *******************************************************************************************/
void w_voting(beacon b[MAX_BEACONS], int num_b, grid g, pathLossModel pl, float *estX, float *estY)
{
	int i;
	float x,y;
	float score;
	float MaxScore = 0;
	int MaxScoreIndex = 0;
	float calcX = 0;
	float calcY = 0;
	float weight;
	float computedDistance;
	float minDistanceBound, maxDistanceBound;
	float * XofCellsWithMaxScore;
	float * YofCellsWithMaxScore;
	float std; //Standard deviation
	int xlen, ylen;
	float c;
	xlen = floor((g.xmax-g.xmin)/g.stepp+1);
	ylen = floor((g.ymax-g.ymin)/g.stepp+1);

	if ((XofCellsWithMaxScore = (float*) malloc(sizeof(float)*xlen*ylen+1)) == NULL)
		printf("X malloc failed\r\n");
	if ((YofCellsWithMaxScore = (float*) malloc(sizeof(float)*xlen*ylen+1)) == NULL)
		printf("Y malloc failed\r\n");
	c = (pl.sigdB*LOG10)/(10*pl.alpha);//90% confidence interval
	for (x = g.xmin; x <= g.xmax; x += g.stepp)
	{
		for (y = g.ymin; y <= g.ymax; y += g.stepp)
		{
			score = 0;
			for (i = 0; i < num_b; i++)
			{
				computedDistance = sqrt(pow((x - b[i].x),2) + pow((y - b[i].y),2));
				std = c*b[i].d;
				minDistanceBound = ((b[i].d - std) > 0) ? (b[i].d - std) : 0;
				maxDistanceBound = b[i].d + std;
				weight = 1/b[i].d;
				if((minDistanceBound <= computedDistance) && (computedDistance <= maxDistanceBound ))
				{
					score+= weight;
				}
			}
			if (score >= MaxScore)
			{
				if (score > MaxScore)
				{
					MaxScore = score;
					MaxScoreIndex = 0;
				}
				if (MaxScore)
				{
					XofCellsWithMaxScore[MaxScoreIndex] = x;
					YofCellsWithMaxScore[MaxScoreIndex] = y;
					MaxScoreIndex++;
				}
			}
		}
	}
	x = 0;
	y = 0;
	for( i = 0; i < MaxScoreIndex; i++)
	{
		x += XofCellsWithMaxScore[i]/(MaxScoreIndex);
		y += YofCellsWithMaxScore[i]/(MaxScoreIndex);
	}
	*estX = x;
	*estY = y;
	free(XofCellsWithMaxScore);
	free(YofCellsWithMaxScore);
}


/*********************************************************************************************
* Title: gradient
*
* Description: This is the gradient descent algorithm. Computes the
*        least-mean-square (LMS), then the derivative of the LMS.
*        Finally it moves towards minimizing the error by moving
*        towards the minimum of the gradient by delta*gradient.
* Author: Jeffrey Vander Stoep jevander@u.washington.edu
*
*********************************************************************************************/
void w_gradient(beacon b[MAX_BEACONS], int num_b, float delta, float * estX, float * estY) //, uint8_t nodeId)
{
	int i,j;
	static int IterationNumber;
	float oldX, oldY;
	float x,y;
	float tempX, tempY;
	float Xgradient = 0, Ygradient = 0;
	float neighborX, neighborY;
	float computedDistance;
	float mindistance; //for initializing to nearest neighbor
	float weight;
	int mindist_index = 0;
	if ((!(*estX)) || (!(*estY))) IterationNumber = 0;
	if (IterationNumber ==0)
	{
		mindistance = b[0].d;
		for (i = 1; i< num_b; i++)
		{
			if (b[i].d < mindistance)
			{
				mindistance = b[i].d;
				mindist_index = i;
			}
		}
		x = b[mindist_index].x + 0.1;
		y = b[mindist_index].y + 0.1;
		IterationNumber++;
	}
	else
	{
		oldX = *estX;
		oldY = *estY;
		for(i=0;i < num_b; i++)
		{
			neighborX = b[i].x;
			neighborY = b[i].y;
			weight = 1.0/b[i].d;
			computedDistance = sqrt(pow((oldX - neighborX),2) + pow((oldY - neighborY),2));
			Xgradient = Xgradient + weight*(computedDistance - b[i].d)* ((oldX - neighborX))/computedDistance;
			Ygradient = Ygradient + weight*(computedDistance - b[i].d)* ((oldY - neighborY))/computedDistance;
		}
		tempX = ((oldX - (delta * Xgradient)));// > 0);// ? (oldX - (ALPHA * Xgradient)) : 0;
		tempY = ((oldY - (delta * Ygradient)));// > 0);// ? (oldY - (ALPHA * Ygradient)) : 0;
		x = tempX;
		y = tempY;
	}
	*estX = x;
	*estY = y;
}
