/*******************************************************************************
*	Localization Algorithms. localization.h
*	Last Modified: 27/May/2009
*	
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*	
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*	
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*	
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#ifndef LOCALIZATION_H
#define LOCALIZATION_H

#include <stdio.h>
#include <math.h>
#include <time.h>

#define MAX_BEACONS 20
#define MAX_RSSI_BEACON 128	// maximum RSSI recordings in the struct beacon
#define MAX_WAIT 10		//	report anchor every MAX_WAIT * LOCAL_TIMER_INTERVAL sec
#define LOG10 2.3026
//#define LOCAL_TIMER_INTERVAL 2	// local timer interval
#define GRID_SIZE 10
#define GRID_STEP 0.1
#define ALPHA 2.7
#define K 0.00000025
#define SIGDB 4
#define DELTA 0.15

// list of beacons can be heard from current node
typedef struct beacon
{
	__u16 addr;
	float x;
	float y;
	int8_t RSSI[MAX_RSSI_BEACON];
	char RSSIindex;
	float d;
} beacon;

//
typedef struct grid
{
	float xmin;
	float xmax;
	float ymin;
	float ymax;
	float stepp;
} grid;

//
typedef struct pathLossModel
{
	float alpha;
	float k;
	float sigdB;
} pathLossModel;

/* localization packet handler */
void localization_msg(packet* in_pkt);
/* timer */
void *local_timer();
/* update localization config file */
int update_local_config();
/* initializes local config file */
int init_local_config();

// localization algorithms
/* centroid */
void centroid(beacon b[MAX_BEACONS], int num_b, float *estX, float *estY);
/* weighted centroid */
void weightedCentroid(beacon b[MAX_BEACONS], int num_b, float *estX, float *estY);
/* gradient */
void gradient(beacon b[MAX_BEACONS], int num_b, float delta, float *estX, float *estY);
/* voting */
void voting(beacon b[MAX_BEACONS], int num_b, grid g, float errorThreshold, float *estX, float *estY);
/* weighted voting */
void w_voting(beacon b[MAX_BEACONS], int num_b, grid g, pathLossModel pl, float *estX, float *estY);
/* weighted gradient */
void w_gradient(beacon b[MAX_BEACONS], int num_b, float delta, float * estX, float * estY);
/* RSSI to distance conversion */
float RSSI_to_distance(float RSSI, float k, float alpha);

#endif
