/*******************************************************************************
*	Packet sending/receiving. network.h
*	Last Modified: 21/May/2009
*
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#ifndef NETWORK_H
#define NETWORK_H

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <linux/ioctl.h>
#include <pthread.h>
#include "tosmac.h"
#include "util.h"

#define MAX_PAYLOAD_SIZE 28	// max size for data payload
#define DATA_SIZE 26		// data payload size (MAX_PAYLOAD_SIZE - 2)
#define CHANNEL 11			// channel used for radio
#define POWER_LEVEL 31		// radio power level (3 to 31)
#define RETRIES 6			// # of retires if packet failes to send
#define ACK_TIMEOUT 2		// time for the sender to wait for ack (in sec)
#define RECV_TIMEOUT 1		// time for the receiver to wait for a packet from sender (in sec)
#define RECV_TIMEOUT_U 800000	// time for the receiver to wait for a packet from sender (in us)
#define WAIT_TIME 20000		// wait time between sending packets (in us)
#define ACK_RESEND 1		// # of ACK resend (default 1)
#define CMD_PKT_SIZE 12		// command packet size (12 bytes)
#define CMD_PKT_TYPE 7		// command packet type (default 7)
#define ANCHOR_BROADCAST 8	// anchor node broadcast packet type (default 8)
#define ROUTING_PKT_TYPE 9	// back to base routing packet type (default 9)
#define TO_NODE_ROUTING_PKT_TYPE 10	// to each node routing packet type (default 10)
#define TO_BASE_DATA_PKT_TYPE 11	// to base localization data packet type (default 11)

// debug switch, if true will print out received packet info
#define network_debug 0

/* incoming packet struct */
typedef struct packet
{
	unsigned char data[DATA_SIZE];	// data payload
	__u16 srcAddr;	// sender address
	int dataSize;	// packet size
	int seq;		// packet sequence #
	char type;		// packet type (port number)
	int8_t rss;		// rssi
} packet;

/* initialize radio */
int cc2420_init(int pwr_level);

/* initialize packet */
int initPkt(packet *Pkt, int type);

/* send ACK */
int sendACK(__u16 destAddr, __u16 localAddr, unsigned char seq);

/* receive ACK */
int recvACK(__u16 destAddr, __u16 localAddr, unsigned char seq);

/* handler receive */
int h_recvPkt(__u16 localAddr, packet *in_pkt, int ACK_en);

/* send function for handler */
int h_send(__u16 destAddr, packet *out_pkt, int ACK_en);

#endif
