/*******************************************************************************
*	Multi-hop router. router.h
*	Last Modified: 21/May/2009
*
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#ifndef ROUTER_H
#define ROUTER_H

#include <time.h>
#include <stdlib.h>
#include "network.h"
#include "util.h"

// multihop router parameters
#define MIN_UPDATE_INTERVAL 2500000	// routing boradcast inteval (in us)
#define ROUTING_TABLE_SIZE 4	// size of to base routing table (max 11)
#define ROUTING_DATA_SIZE 26	// ROUTING_DATA_SIZE = ROUTING_TABLE_SIZE * 5 + 6
#define MAX_STAY 10				// maximum time an entry stays in routing table
#define MAX_STAY_TO_NODE 40		// maximum time an entry stays in to-node routing table
#define BROADCAST_ADDR 0xffff	// broadcasting address
// other parameters
#define MAX_NUM_NODES_IN_RT 7	//
#define TO_BASE_ROUTING_RESEND 1	// # of re-sends for to base routing packet
#define TO_NODE_ROUTING_RESEND 1	// # of re-sends for to node routing packet
#define TABLE_DISPLAY 5			// display and report routing table to GUI every TABLE_DISPLAY sec

/* node property */
typedef struct nodeProperty
{
	__u16 addr;	// local address
	__u8 numHops;		// # of hops to base
} nodeProperty;

/* routing table entry */
typedef struct routingEntry
{
	__u16 nextHop;	// address for next hop
	__u8 numHops;			// # of hops to base
	int lastHeard;			// when is it last updated
} routingEntry;

/* base routing table entry */
typedef struct baseRoutingEntry
{
	int size;	// size of the address list (including the dest node)
	__u16 destAddr;	// address of destination node
	__u16 *addrList;	// list of addresses to get to destination node
	int lastHeard;			// when is it last updated (not used)
} baseRoutingEntry;

/* linked list node for base routing table */
typedef struct node_St
{
	baseRoutingEntry e;
	struct node_St *next;
} Node;

/* function to append node to the linked list*/
Node *append(Node *h, baseRoutingEntry e);

/* this funtion returns the routing table entry for given address */
baseRoutingEntry *return_entry(Node* h, __u16 addr);

/* this function deletes node from the linked list */
int delete_node(Node *prev, Node *cur);

/* checks and remove old to-node entries */
int check_node(Node *h);

/* initialize routing table */
int initTable();

/* send to base routing beacon */
int send_to_base_rtPkt();

/* send to node routing packet */
int send_to_node_rtPkt();

/* warpper for sending routing beacons */
void *routingBeacon();

/* receive routing beacon & update routing table worker*/
int recv_rtPkt(packet *in_pkt);

/* update routing table */
int updateTable(nodeProperty src_node, __u16 *canHear);

/* send data back to base node via routing */
int sendtoBase(packet *out_pkt, int ACK_en);

/* send data to nodes */
int sendtoNode(__u16 destAddr, packet *out_pkt, int ACK_en);

/* forward routing packets to base */
int forward_rt_toBase(packet *out_pkt);

/* receive base-to-node routing packets at base */
int base_recv_rtPkt(packet *in_pkt);

/* helper thread to display routing table periodcally */
void *display_routing_table();

#endif
