/*******************************************************************************
*	Socket functions. socket.c
*	Last Modified: 1/May/2009
*
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#include "socket.h"

#define socket_debug 0

// some global vars
struct sockaddr_in s_client; //socket info about machine connecting
struct sockaddr_in s_server; //
int socksize = sizeof(struct sockaddr_in);
int s = 0;			//
char *GUI_ADDR = GUI_IP;	
char GUI_DATA = 0;	//
char GUI_ACK = 1;	//
char GUI_PROG = 2;	//
char GUI_DONE = 3;	//
char GUI_ERROR = 4;	//
char GUI_ROUTING = 5;

/*
 * this function receives list of commands to be sent from the GUI
 * it is run as a seperate thread constantly waiting to receive commands
 * inputs:
 *	cmd_list: list of commands to be sent
 * outputs:
 *	N/A
 */
void *cmdSocket(unsigned char *cmd_list)
{
	unsigned char buf[BUFFER_LEN];	// socket data buffer
	extern int sendCommand;				// command flag
	extern int doneSending;				// done sending flag
	extern pthread_mutex_t flag_mutex;	// flag mutex
	extern char total_cmds;	// total number of cmds to be sent
	extern char sent_cmds;	// number of cmds sent
	
	printf("Listening for command packet from GUI.\n\n");
	// Socket setup
	if (( s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
	{
		perror("\nFailed to create CMD socket\n\n");
		exit(1);
	}
	memset(&s_server, 0, sizeof(s_server));	// zero out server struct before filling out again
	s_server.sin_family = AF_INET;			// Set to TCP/IP
	s_server.sin_addr.s_addr = INADDR_ANY;	// Allow any interface
	s_server.sin_port = htons(CMD_PORT);	// set server to port PORT;
	int binderr = bind(s, (struct sockaddr *)&s_server, socksize);
	if (binderr < 0)
	{
		perror("\nBind error\n\n");
		exit(-1);
	}
	// Set up client socket (IP address and port # of GUI is hardcoded here)
	memset(&s_client, 0, sizeof(s_client)); // zero out destination struct before filling out again
	s_client.sin_family = AF_INET;			// Set to TCP/IP
	s_client.sin_addr.s_addr = inet_addr(GUI_ADDR);	// connect to GUI by default
	s_client.sin_port = htons(DATA_PORT);	// set client port # to DATA_PORT;
	//
	while(1)
	{
		//client instantiates
		if (!sendCommand) {
			// send DONE msg back to GUI
			if (doneSending) {
				if (sendto(s, &GUI_DONE, 1, 0, (struct sockaddr *)&s_client, socksize) == -1)
					printf("Send back to GUI failed!\n");
			}
			// receive cmds from GUI
			if (recvfrom(s, buf, BUFFER_LEN, 0, (struct sockaddr *)&s_client, &socksize) != -1)
			{
				//connected = 1;
				fprintf(stderr, "Received packet from %s:%d\nData: %s\n\n", inet_ntoa(s_client.sin_addr), ntohs(s_client.sin_port), buf);
				// done receving data from GUI, send cmds out
				pthread_mutex_lock(&flag_mutex);
				//doneSending = 0;	// reset flag for done sending cmds
				sendCommand = 1;	// set flag for sending commands
				memcpy(cmd_list, buf, BUFFER_LEN);	// get cmd list
				pthread_mutex_unlock(&flag_mutex);
			}
			// update # of cmds to be sent
			total_cmds = buf[0];
			//printf("message = %d %x %s\n", buf[0], buf[1], buf[3]);
			
			// send ACK back to GUI
			if (sendto(s, &GUI_ACK, 1, 0, (struct sockaddr *)&s_client, socksize) == -1)
				printf("Send back to GUI failed!\n");
		}
		else {
			if (socket_debug) {
				printf("Sending commands!\n");
				printf("Total # of commands is: %d, %d sent.\n", total_cmds, sent_cmds);
			}
			// build respond packet to GUI
			memcpy(buf, &GUI_PROG, 1);
			memcpy(buf + 1, &sent_cmds, 1);
			memcpy(buf + 2, &total_cmds, 1);
			//send prograss pakcet back to GUI
			if (sendto(s, buf, 3, 0, (struct sockaddr *)&s_client, socksize) == -1)
				printf("Send cmd sending progress back to GUI failed!\n");
			sleep(UPDATE_INTEVAL);	// wait a little before receiving next command stream
		}
	}
	close(s);	// close socket
	pthread_exit(NULL);	// exit thread
}


/*
 * this function sends the localization data back to the GUI via socket
 * inputs:
 *	in_pkt: data packet to be reported
 * outputs:
 * returns 0 if successful -1 otherwise
 */
int dataSocket(packet in_pkt)
{
	unsigned char buf[CMD_PKT_SIZE + 1];	// socket data buffer
	
	printf("\nSending localization data to GUI.\n\n");
	
	// build packet content
	memcpy(buf, &GUI_DATA, 1);
	memcpy(buf + 1, in_pkt.data, CMD_PKT_SIZE);
	
	//send localization data back to GUI
	if (sendto(s, buf, CMD_PKT_SIZE + 1, 0, (struct sockaddr *)&s_client, socksize) == -1) {
		printf("Send localization data back to GUI failed!\n");
		return -1;
	}
	
	if (socket_debug) {
		int i = 0;
		for (i = 0; i <= CMD_PKT_SIZE + 1; i++) {
			printf("Data packet byte %d is %d\n", i, buf[i]);
		}
		printf("\n\n");
	}
	return 0;
}


/*
* this function sends the packet failed to send back to GUI
* inputs:
*	in_pkt: data packet to be reported
* outputs:
* returns 0 if successful -1 otherwise
*/
int errorSocket(packet in_pkt)
{
	unsigned char buf[CMD_PKT_SIZE + 1];	// socket data buffer
	
	printf("\nSending error packet to GUI.\n\n");
	
	// build packet content
	memcpy(buf, &GUI_ERROR, 1);
	memcpy(buf + 1, in_pkt.data, CMD_PKT_SIZE);
	
	//send localization data back to GUI
	if (sendto(s, buf, CMD_PKT_SIZE + 1, 0, (struct sockaddr *)&s_client, socksize) == -1) {
		printf("Send error packet back to GUI failed!\n");
		return -1;
	}
	return 0;
}


/*
* this function sends the to nodes routing table back to GUI
* inputs:
*	routingTable: to nodes routing tale in string form
*	routingTable_size: # of entries in routingTable
* outputs:
* returns 0 if successful -1 otherwise
*/
int routingSocket(__u16 *routingTable, unsigned char routingTable_size)
{
	unsigned char buf[BUFFER_LEN];	// socket data buffer
	
	printf("\nSending routing table to GUI.\n\n");
	
	// build packet content
	memcpy(buf, &GUI_ROUTING, 1);
	memcpy(buf + 1, &routingTable_size, 1);
	memcpy(buf + 2, routingTable, routingTable_size * 2);
	
	//send localization data back to GUI
	if (sendto(s, buf, BUFFER_LEN, 0, (struct sockaddr *)&s_client, socksize) == -1) {
		printf("Send localization data back to GUI failed!\n");
		return -1;
	}
	return 0;
}
