/*******************************************************************************
*	Utility functions. util.c
*	Last Modified: 27/May/2009
*
*	Copyright (C) 2009 Tony Wu mdzz@u.washington.edu
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/

#include "util.h"

// global vars
__u16 localAddr = 0;	// local address for current node
int isBase = 0;	// base flag
T_Node *tNode_head = NULL;	//
pthread_mutex_t joinList_mutex = PTHREAD_MUTEX_INITIALIZER;	//

/*
 * helper function sets the current node address according to the config file
 */
int set_addr()
{
	FILE *fp;
	// read local address form config file
	fp = fopen("/root/router.conf", "r");
	if (fp == NULL) {
		printf("Router config file missing.\n");
		return -1;
	}
	else {
		fscanf(fp, "%x\n%d\n", &localAddr, &isBase);
		fclose(fp);		// close file
	}
	return 0;
}


/*
 * helper function sets the current node address according to the config file
 * NOTICE must run set_addr() first before calling this function
 */
__u16 get_addr()
{
	return localAddr;
}


/*
 * this function checks if the current node is the base node
 * NOTICE must run set_addr() first before calling this function
 * output:
 * 	0 not, 1 yes
 */
int check_base()
{
	return isBase;
}


/*
 * this function initializes the join list
 */
int init_joinList()
{
	pthread_t e;	// dummy element for head of the list
	tNode_head = tNode_append(tNode_head, e);
	return 0;
}


/*
 * this function addes a node to the linked list.
 * inputs:
 * 	h: head of list
 * outputs:
 *	p: pointer to node added
 */
T_Node *tNode_append(T_Node *h, pthread_t e)
{
	T_Node *prev = h;
	T_Node *p = h;
	int update = 0;	// flag for updating internal node
	
	// go to end of list
	pthread_mutex_lock(&joinList_mutex);
	if (h)
		p = p->next;
	while (p) {
		prev = p;
		p = p->next;
	}
	
	p = (T_Node *)malloc(sizeof(T_Node));
	p->e = e;
	p->next = NULL;
	if (prev)	//
		prev->next = p;
	pthread_mutex_unlock(&joinList_mutex);
	
	if (util_debug)
		printf("\nAdding thread: %d\n\n", p->e);
	
	return p;
}


/*
 * this function deletes a node form the join list
 */
void tNode_delete(T_Node *prev, T_Node *cur)
{
	//pthread_mutex_lock(&joinList_mutex);
	prev->next = cur->next;	// link to next
	free(cur);				// free memory
	//pthread_mutex_unlock(&joinList_mutex);
	return;
}


/*
 * this function returns tNode_head
 */
T_Node *return_tNode_head()
{
	return tNode_head;
}


/*
 * this function joins the given threads to the main thread
 */
void join_threads()
{
	T_Node *cur = tNode_head;	// get join list head, skip first element
	T_Node *tmp = NULL;
	
	if (util_debug)
		printf("\nJoining packet handler threads!\n");
	
	pthread_mutex_lock(&joinList_mutex);
	if (cur)
		cur = cur->next;
	while (cur) {
		if (util_debug)
			printf("Joining thread: %d\n\n", cur->e);
		tmp = cur;
		cur = cur->next;
		pthread_join(tmp->e, NULL);		// join
		tNode_delete(tNode_head, tmp);	// delete used thread ID
	}
	pthread_mutex_unlock(&joinList_mutex);
	return;
}
